HANDLE WINAPI CreateFileA(LPTSTR lpFileName, DWORD dwDesiredAccess, DWORD dwShareMode, LPVOID lpSecurityAttributes, DWORD dwCreationDisposition, DWORD dwFlagsAndAttributes, HANDLE hTemplateFile);
BOOL WINAPI GetFileSizeEx(HANDLE hFile, PLARGE_INTEGER lpFileSize);
BOOL WINAPI SetFileInformationByHandle(HANDLE hFile, DWORD FileInformationClass, LPVOID lpFileInformation, DWORD dwBufferSize);
BOOL WINAPI ReadFile(HANDLE hFile, LPVOID lpBuffer, DWORD nNumberOfBytesToRead, LPDWORD lpNumberOfBytesRead, LPVOID lpOverlapped);
BOOL WINAPI WriteFile(HANDLE hFile, LPVOID lpBuffer, DWORD nNumberOfBytesToWrite, LPDWORD lpNumberOfBytesWritten, LPVOID lpOverlapped);
BOOL WINAPI FlushFileBuffers(HANDLE hFile);
BOOL WINAPI CloseHandle(HANDLE hObject);

const HANDLE INVALID_HANDLE = (HANDLE) -1;

//#define UNBUFFERED_DISK

#define FILE_READ_DATA           0x00000001
#define FILE_LIST_DIRECTORY      0x00000001
#define FILE_WRITE_DATA          0x00000002
#define FILE_ADD_FILE            0x00000002
#define FILE_APPEND_DATA         0x00000004
#define FILE_ADD_SUBDIRECTORY    0x00000004
#define FILE_READ_EA             0x00000008
#define FILE_WRITE_EA            0x00000010
#define FILE_EXECUTE             0x00000020
#define FILE_TRAVERSE            0x00000020
#define FILE_DELETE_CHILD        0x00000040
#define FILE_READ_ATTRIBUTES     0x00000080
#define FILE_WRITE_ATTRIBUTES    0x00000100
#define SPECIFIC_RIGHTS_ALL      0x0000FFFF
#define DELETE                   0x00010000
#define READ_CONTROL             0x00020000
#define STANDARD_RIGHTS_READ     0x00020000
#define STANDARD_RIGHTS_WRITE    0x00020000
#define STANDARD_RIGHTS_EXECUTE  0x00020000
#define WRITE_DAC                0x00040000
#define WRITE_OWNER              0x00080000
#define STANDARD_RIGHTS_REQUIRED 0x000F0000
#define SYNCHRONIZE              0x00100000
#define STANDARD_RIGHTS_ALL      0x001F0000
#define ACCESS_SYSTEM_SECURITY   0x01000000
#define MAXIMUM_ALLOWED          0x02000000
#define GENERIC_ALL              0x10000000
#define GENERIC_EXECUTE          0x20000000
#define GENERIC_WRITE            0x40000000
#define GENERIC_READ             0x80000000

#define FILE_SHARE_NONE   0
#define FILE_SHARE_READ   1
#define FILE_SHARE_WRITE  2
#define FILE_SHARE_DELETE 4

#define CREATE_NEW        1
#define CREATE_ALWAYS     2
#define OPEN_EXISTING     3
#define OPEN_ALWAYS       4
#define TRUNCATE_EXISTING 5

#define FILE_ATTRIBUTE_READONLY            0x00000001
#define FILE_ATTRIBUTE_HIDDEN              0x00000002
#define FILE_ATTRIBUTE_SYSTEM              0x00000004
#define FILE_ATTRIBUTE_DIRECTORY           0x00000010
#define FILE_ATTRIBUTE_ARCHIVE             0x00000020
#define FILE_ATTRIBUTE_DEVICE              0x00000040
#define FILE_ATTRIBUTE_NORMAL              0x00000080
#define FILE_ATTRIBUTE_TEMPORARY           0x00000100
#define FILE_ATTRIBUTE_SPARSE_FILE         0x00000200
#define FILE_ATTRIBUTE_REPARSE_POINT       0x00000400
#define FILE_ATTRIBUTE_COMPRESSED          0x00000800
#define FILE_ATTRIBUTE_OFFLINE             0x00001000
#define FILE_ATTRIBUTE_NOT_CONTENT_INDEXED 0x00002000
#define FILE_ATTRIBUTE_ENCRYPTED           0x00004000
#define FILE_ATTRIBUTE_VIRTUAL             0x00010000
#define FILE_FLAG_FIRST_PIPE_INSTANCE      0x00080000
#define FILE_FLAG_OPEN_NO_RECALL           0x00100000
#define FILE_FLAG_OPEN_REPARSE_POINT       0x00200000
#define FILE_FLAG_POSIX_SEMANTICS          0x01000000
#define FILE_FLAG_BACKUP_SEMANTICS         0x02000000
#define FILE_FLAG_DELETE_ON_CLOSE          0x04000000
#define FILE_FLAG_SEQUENTIAL_SCAN          0x08000000
#define FILE_FLAG_RANDOM_ACCESS            0x10000000
#define FILE_FLAG_NO_BUFFERING             0x20000000
#define FILE_FLAG_OVERLAPPED               0x40000000
#define FILE_FLAG_WRITE_THROUGH            0x80000000

#define FILE_INFORMATION_LENGTH 0x00000006

HANDLE OpenFileForRead(LPTSTR filepath) {
#ifndef UNBUFFERED_DISK
	return CreateFileA(filepath, FILE_READ_DATA | STANDARD_RIGHTS_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
#else
	return CreateFileA(filepath, FILE_READ_DATA | STANDARD_RIGHTS_READ, FILE_SHARE_NONE, NULL, OPEN_EXISTING, FILE_FLAG_NO_BUFFERING, NULL);
#endif
};
HANDLE CreateFileForWrite(LPTSTR filepath) {
#ifndef UNBUFFERED_DISK
	return CreateFileA(filepath, FILE_WRITE_DATA | STANDARD_RIGHTS_WRITE, FILE_SHARE_NONE, NULL, CREATE_ALWAYS, 0, NULL);
#else
	return CreateFileA(filepath, FILE_WRITE_DATA | STANDARD_RIGHTS_WRITE, FILE_SHARE_NONE, NULL, CREATE_ALWAYS, FILE_FLAG_WRITE_THROUGH, NULL);
#endif
};
PBUFFERED_FILE InitBufferedFile(HANDLE hFile, DWORD length) {
	PBUFFERED_FILE nbf = (PBUFFERED_FILE) MemAlloc(sizeof(BUFFERED_FILE));
	nbf->hFile = hFile;
	nbf->bufferSize = length;
	nbf->bufferPos = 0;
	nbf->bufferFill = 0;
	nbf->buffer = (BYTE*) MemAlloc(length);
	return nbf;
};
int ReadFileByte(HANDLE hFile) {
	BYTE rval = 0;
	BOOL rret = FALSE;
	DWORD rlen = 0;
	rret = ReadFile(hFile, &rval, 1, &rlen, NULL);
	if ((rret == TRUE) && (rlen != 0)) {
		return rval;
	} else {
		return -1;
	};
};
void WriteFileByte(HANDLE hFile, BYTE val) {
	DWORD tmpl = 0;
	WriteFile(hFile, &val, 1, &tmpl, NULL);
};
int ReadBufferedByte(PBUFFERED_FILE file) {
	int retb = 0;
	if(file->bufferPos >= file->bufferFill) { 
		ReadFile(file->hFile, file->buffer, file->bufferSize, &(file->bufferFill), NULL);
		file->bufferPos = 0;
		if(file->bufferFill == 0) { return -1; };
	};
	retb = file->buffer[file->bufferPos];
	file->bufferPos++;
	return retb;
};
void WriteBufferedByte(PBUFFERED_FILE file, BYTE val) {
	if(file->bufferPos >= file->bufferSize) { 
		WriteFile(file->hFile, file->buffer, file->bufferPos, &file->bufferFill, NULL);
		file->bufferPos = 0;
	};
	file->buffer[file->bufferPos] = val;
	file->bufferPos++;
};
void FlushBufferedFile(PBUFFERED_FILE file, BOOL force_cache_flush) {
	WriteFile(file->hFile, file->buffer, file->bufferPos, &file->bufferFill, NULL);
	file->bufferPos = 0;
	if(force_cache_flush == TRUE) { FlushFileBuffers(file->hFile); }
};
void CloseBufferedFile(PBUFFERED_FILE file) {
	CloseHandle(file->hFile);
	MemFree(file->buffer);
	MemFree(file);
	file = NULL;
};
void SetFileLength(HANDLE file, PLARGE_INTEGER length) {
	SetFileInformationByHandle(file, FILE_INFORMATION_LENGTH, length, sizeof(LARGE_INTEGER));
};